/**
 * Created by buns on 1/9/16.
 *
 * 开启业务点的运行
 *  1.先获取根节点的业务点并启动它
 *  2. 条件判断节点：node-switch
 *         判断当前是否在end节点中,
 *              如果在则直接进行结果输出
 *              如果不在则判断上一个节点的参数数据是否能够匹配到跳转节点,匹配到了直接进行下一个节点的输出，反之则输出无法匹配的业务线中断信息
 *     处理器节点：node-process
 *         判断当前是否在end节点中,
 *              如果在则直接进行结果输出
 *              如果不在则调用控制器,然后判断是否有下一个节点
 *     并行节点: node-parallel
 *         判断当前是否在end节点中,
 *              如果在则直接进行结果输出
 *              如果不在则获取next节点的结合，然后分法出去
 *     合并节点: node-combine
 *         判断当前是否在end节点中,
 *              如果在则直接进行结果输出
 *              如果不在则将结果汇总之后输出到下一个节点
 */

"use strict";

var Promise = require("bluebird");
var util = require("util");
var events = require("events");//EventEmitter通过events模块来访问

var store = require("../store");
var eRunState = require("../model/po-eRunState");

var switchNode = require("./node-switch");
var parallelNode = require("./node-parallel");
var processNode = require("./node-process");
var combineNode = require("./node-combine");

function Line() {
    this.state = eRunState.prepare;
    this.directive = null;
    this.args = null;
    this.requestArgs = null;
    this.promise = null;
    events.EventEmitter.call(this);
}

util.inherits(Line, events.EventEmitter);

/**
 * 执行业务流
 *  1.获取root业务节点，开始执行
 *  2.分析每一个节点的类型
 *      null/undefined/normal: 默认类型--业务操作类型的节点
 *      switch: 判断节点
 *      parallel:并行节点
 *      combine: 合并节点
 *    针对每种类型的参数判断和参数处理，如果出现错误则直接catch抛出，并记录下日志，然后有
 */
Line.prototype.run = function run() {
    var self = this,
        args = self.args,
        lineConfig;

    self.state = eRunState.doing;

    self.context = lineConfig = store.getBusiLineConfig(self.directive);

    var node = self.curNode = lineConfig.nodes[lineConfig.root];

    self.promise =
        Promise.resolve(args)
            .then((function ($line, $node) {
                $line.curNode = $node;
                return function (data) {
                    var result;
                    switch ($node.type) {
                        case "switch":
                            result = switchNode.run($line, $node, data);
                            break;
                        case "parallel":
                            result = parallelNode.run($line, $node, data);
                            break;
                        case "combine":
                            result = combineNode.run($line, $node, data);
                            break;
                        default :
                            result = processNode.run($line, $node, data);
                            break;
                    }
                    return (result instanceof Error)
                        ? Promise.reject(result)
                        : result;
                };
            }(self, node)))
            .then((function (_line) {
                return function (data) {
                    return _line.isCombineNodeAppearance
                        ? data[data.length - 1]
                        : data;
                }
            }(self)));

    return self;
};

module.exports = Line;