"use strict";

var Line = require("./line");
var eRunState = require("../model/po-eRunState");
var objUtil = require("../utils/util-object");

var caches = {};

function LinePool() {
    this.maxSize = 500;
    this.maxListener = 1000;
}

/**
 * 全程控制业务流水对象的执行和分配
 *
 * 判断当前缓冲区中业务流处理对象的状态
 *  1.如果是prepare，则将该对象的参数、指令等属性重置
 *  2.如果是doing,匹配两个对象的指令参数是否一致
 *      2.1 一致: 则将该客户端的回调填充到主题end中
 *      2.2 不一致: 则重新生成一个业务流处理对象，添加至buffer区域，然后执行;
 *  3.如果没有找到，则重新生成一个业务流处理对象，添加至buffer区域,然后执行;
 * @param directive 业务执行指令
 * @param directiveArgs 业务指令携带的参数对象
 * @returns {*}
 */
LinePool.prototype.get = function (directive, directiveArgs) {
    var i,
        len,
        line,
        self = this;

    var lines = caches[directive] || (caches[directive] = []);

    len = lines.length;

    if (len === self.maxSize) {
        lines = caches[directive] = [];
    }

    for (i = 0; i < len; i++) {
        line = lines[i];
        if (!line) continue;
        if (line.state === eRunState.prepare) {
            line.args = directiveArgs;
            line.requestArgs = objUtil.copy(directiveArgs);
            line.removeAllListeners();
            line.once("completed", initState);
            return line;
        } else if (line.state === eRunState.doing) {
            if (line.listenerCount('end') === self.maxListener) {
                break;
            }
            if (objUtil.compare(line.requestArgs, directiveArgs)) {
                return line;
            }
        }
    }
    line = new Line();
    line.setMaxListeners(self.maxListener);
    line.state = eRunState.prepare;
    line.args = directiveArgs;
    line.requestArgs = objUtil.copy(directiveArgs);
    line.directive = directive;
    line.removeAllListeners();
    line.once("completed", initState);
    lines.push(line);
    return line;

    function initState(busiFlow) {
        busiFlow.state = eRunState.prepare;
    }
};

LinePool.prototype.setMaxSize = function (val) {
    this.maxSize = val || 1000;
};

LinePool.prototype.setMaxListeners = function (val) {
    this.maxListener = val || 1000;
};


module.exports = {
    instance: new LinePool()
};