"use strict";

var Promise = require("bluebird");
var util = require("util");

var error = require("../error");

/**
 *
 * @type {number}
 */
var handledPromiseCount = 0;

/**
 * 输出到下一个Promise的数据字典
 * @type {{}}
 */
var outDataDic = {};

/**
 * 判断进行合并的是单个点还是多个点
 * @param $curNode
 * @returns {number}
 */
function isMultiple($curNode) {
    if (!$curNode.prev) return 1;
    return $curNode.prev.split(',').length;
}

/**
 * 重组参数
 * @param dataRule
 * @param data
 * @param nodeId
 * @returns {*}
 */
function recombination(dataRule, data, nodeId) {
    var rule,
        propArr,
        prop,
        val,
        idx = 0,
        len;
    if (nodeId) {
        rule = dataRule[nodeId];
        if (!rule) {
            return new Error("Can't Find Error:");
        }
        val = data;
        propArr = rule.split('.');
        len = propArr.length;
        prop = propArr[idx];
        while (prop) {
            if (val.hasOwnProperty(prop)) {
                val = val[prop];
            } else {
                return new Error("Can't Find Property Error:");
            }
            if (idx === len - 1) {
                break;
            }
            prop = propArr[++idx];
        }
        outDataDic[prop] = val;
    }
    return true;
}

function CombineNode() {
}

/**
 * 处理器业务节点处理
 * @param $line 当前业务线上下文对象
 * @param $node 当前业务点
 * @param $prevOutData 上一个业务点输出的值
 * @param $prevNode 上一个业务点
 */
CombineNode.prototype.run = function ($line, $node, $prevOutData, $prevNode) {
    var promise, result, isMulti, nextNode;
    try {
        $line.isCombineNodeAppearance = true;
        result = recombination($node.dataRule, $prevOutData, $prevNode.id);
        if (result instanceof Error) {
            promise = Promise.reject(result);
        } else {
            isMulti = isMultiple($node);
            if (isMulti === 1) {
                promise = Promise.resolve(outDataDic);
                outDataDic = {};
            } else {
                handledPromiseCount += 1;
                if (handledPromiseCount === isMulti) {
                    handledPromiseCount = 0;
                    if ($node.next) {
                        nextNode = $line.context.nodes[$node.next];
                        if (!nextNode) {
                            return error.errorBuild(error.eErrorType.NotFindNextNode, $line, $node);
                        }
                        promise = Promise
                            .resolve(outDataDic)
                            .then((function (_line, _node, _this, _prevNode) {
                                return function (data) {
                                    var result, typeNode;
                                    switch (_node.type) {
                                        case "switch":
                                            typeNode = require("./node-switch");
                                            result = typeNode.run(_line, _node, data);
                                            break;
                                        case "parallel":
                                            typeNode = require("./node-parallel");
                                            result = _this.run(_line, _node, data);
                                            break;
                                        case "combine":
                                            result = _this.run(_line, _node, data, _prevNode);
                                            break;
                                        default :
                                            typeNode = require("./node-process");
                                            result = typeNode.run(_line, _node, data);
                                            break;
                                    }
                                    if (result instanceof Error) {
                                        return Promise.reject(result);
                                    } else {
                                        return result;
                                    }
                                }
                            }($line, nextNode, this, $node)))

                    } else {
                        /**
                         * 在end节点中
                         */
                        if ($line.context.end && $line.context.end.indexOf($node.id) > -1) {
                            promise = Promise.resolve(outDataDic);
                            outDataDic = {};
                        } else {
                            outDataDic = {};
                            return error.errorBuild(error.eErrorType.NotSetNodeNext, $line, $node);
                        }
                    }
                }
            }
        }
    } catch (e) {
        outDataDic = {};
        return e;
    }
    return promise;
};

var instance = new CombineNode();

module.exports = instance;