"use strict";

var Promise = require("bluebird");
var util = require("util");

var objUtil = require("../utils/util-object");

function ParallelNode() {
}

/**
 * 并发业务节点处理
 * @param $line 当前业务线上下文对象
 * @param $node 当前业务节点
 * @param $prevOutData 上一个业务点输出的值
 */
ParallelNode.prototype.run = function ($line, $node, $prevOutData) {
    var self = this;
    var node = $node;
    var nextNodes, nextNode;
    if (node.next) {
        nextNodes = node.next.split(',');
        if (nextNodes.length === 1) {
            nextNode = $line.context.nodes[node.next];
            if (!nextNode) {
                return new Error("NextNode Can't Find Error:当前业务线(" + $line.directive + ")的并行节点(" + node.id + "),在执行上一个流程输出结果的判断后得到的next节点(" + node.next + ")无法找到，导致流程线中断!");
            }
            return Promise.resolve($prevOutData).then(getPromise($line, nextNode, self));
        } else if (nextNodes.length > 1) {
            for (var i = 0, l = nextNodes.length; i < l; i++) {
                nextNode = $line.context.nodes[nextNodes[i]];
                if (!nextNode) {
                    return new Error("Can't Find Node Error: 业务线(" + $line.directive + ")的并行节点的下一个节点(" + nextNode + ")未找到");
                }
                nextNodes[i] = nextNode;
            }
            return Promise.map(nextNodes, function (node, index) {
                return Promise.resolve(
                    getPromise(
                        $line,
                        node,
                        self,
                        node,
                        index === 0 ? $prevOutData : objUtil.copy($prevOutData)));
            });
        }
    } else {
        /**
         * 在end节点中
         */
        if ($line.context.end && $line.context.end.indexOf(node.id) > -1) {
            return Promise.resolve($prevOutData);
        } else {
            return new Error("NextNode IS NULL Error:当前业务线(" + $line.directive + ")的并行节点(" + node.id + "),在执行上一个流程输出结果的判断后，无法找到下一批执行节点,从而导致流程线中断！");
        }
    }

    function getPromise(_line, _node, _this, _prevNode, data) {
        var result, typeNode;
        switch (_node.type) {
            case "switch":
                typeNode = require("./node-switch");
                result = typeNode.run(_line, _node, data);
                break;
            case "parallel":
                result = _this.run(_line, _node, data);
                break;
            case "combine":
                typeNode = require("./node-combine");
                result = typeNode.run(_line, _node, data, _prevNode);
                break;
            default :
                typeNode = require("./node-process");
                result = typeNode.run(_line, _node, data);
                break;
        }
        if (result instanceof Error) {
            return Promise.reject(result);
        } else {
            return result;
        }
    }
};

var instance = new ParallelNode();

module.exports = instance;
