"use strict";

var Promise = require("bluebird");
var util = require("util");

var store = require("../store");
var error = require("../error");

function ProcessNode() {
}
/**
 * 验证必填参数是否缺失
 * @param line
 * @param node
 * @param data
 * @returns {*}
 */
ProcessNode.prototype.valid = function (line, node, data) {
    var inP = line.context.inP[node.id];
    if (!inP) return true;
    var l = inP.length;
    for (; l--;) {
        if (inP[l].opt) {
            if (!data.hasOwnProperty(i)) {
                return error.errorBuild(error.eErrorType.NotInputNodeRequireParam, line, node, inP);
            }
        }
    }
    return true;
};

/**
 * 处理器业务节点处理
 * logic:
 *         判断当前是否在end节点中,
 *              如果在则直接进行结果输出
 *              如果不在则调用控制器,然后判断是否有下一个节点
 * @param $line 当前业务线上下文对象
 * @param $node 当前执行的业务点
 * @param $prevOutData 上一个业务点输出的值
 */
ProcessNode.prototype.run = function ($line, $node, $prevOutData) {
    var node = $node,
        processor,
        isValid,
        nextNode;

    /**
     * 获取业务点的逻辑处理器
     * @type {*}
     */
    processor = store.getNodeProcessor(node.classPath, $line.directive, $node.id);
    if (!processor) return error.errorBuild(error.eErrorType.NotFindProcessor, $line, $node);

    isValid = this.valid($line, node, $prevOutData);
    if (isValid instanceof Error) {
        return isValid;
    }
    if (node.next) {
        nextNode = $line.context.nodes[node.next];

        if (!nextNode) {
            return error.errorBuild(error.eErrorType.NotFindNextNode, $line, node);
        }
        return new Promise((function (_processor, _data) {
            return function (resolve, reject) {
                _processor.paramDic = _data;
                try {
                    _processor.process(resolve, reject);
                } catch (e) {
                    reject(e);
                }
            }
        }(processor, $prevOutData)))
            .then((function (_line, _node, _this, _prevNode) {
                return function (data) {
                    var result, typeNode;
                    switch (_node.type) {
                        case "switch":
                            typeNode = require("./node-switch");
                            result = typeNode.run(_line, _node, data);
                            break;
                        case "parallel":
                            typeNode = require("./node-parallel");
                            result = typeNode.run(_line, _node, data);
                            break;
                        case "combine":
                            typeNode = require("./node-combine");
                            result = typeNode.run(_line, _node, data, _prevNode);
                            break;
                        default :
                            result = _this.run(_line, _node, data);
                            break;
                    }
                    if (result instanceof Error) {
                        return Promise.reject(result);
                    } else {
                        return result;
                    }
                };
            }($line, nextNode, this, node)));
    } else {
        /**
         * 在end节点中
         */
        if ($line.context.end && $line.context.end.indexOf(node.id) > -1) {
            return new Promise((function (_processor, _data) {
                return function (resolve, reject) {
                    _processor.paramDic = _data;
                    try {
                        _processor.process(resolve, reject);
                    } catch (e) {
                        reject(e);
                    }
                }
            }(processor, $prevOutData)))
        } else {
            return error.errorBuild(error.eErrorType.NotSetNodeNext, $line, $node);
        }
    }
};

var instance = new ProcessNode();

module.exports = instance;
