"use strict";

var Promise = require("bluebird");
var util = require("util");

var error = require("../error");

function SwitchNode() {
}

/**
 * 主要执行数据的验证以及分支条件的判断，最后返回执行的分支节点名称;
 *  如果条件未设置则返回true
 *  如果设置了但是是零匹配,则返回false,反之匹配成功则返回要跳转的节点Id;
 * @param cases
 * @param data
 */
SwitchNode.prototype.valid = function (cases, data) {
    var _data;

    if (!cases) {
        return true;
    }

    _data = data;
    var conditions, l;

    for (var item in cases) {
        if (!cases.hasOwnProperty(item)) continue;
        conditions = cases[item];
        if (!(l = conditions.length)) continue;
        for (; l--;) {
            if (each(conditions[l]) === false) continue;
            return item;
        }
    }
    return false;

    function each(it) {
        var l = it.length, value, result = true;

        for (; l--;) {

            if (!_data.hasOwnProperty(it[l])) continue;

            value = _data[it[l]];

            switch (it[l].symbol) {
                case "eq":
                    result = result && (it[l].value === value);
                    break;
                case "neq":
                    result = result && (value !== it[l].value);
                    break;
                case "gt":
                    result = result && (value > it[l].value);
                    break;
                case "gte":
                    result = result && (value >= it[l].value);
                    break;
                case "lt":
                    result = result && (value < it[l].value);
                    break;
                case "lte":
                    result = result && (value <= it[l].value);
                    break;
                case "in":
                    result = result && (it[l].value.indexOf(value) > -1);
                    break;
                case "nin":
                    result = result && (it[l].value.indexOf(value) === -1);
                    break;
                default:
                    return false;
            }
            if (!result) return false;
        }
        return result;
    }
};

/**
 * 执行条件判断业务节点
 * @param $line
 * @param $node
 * @param $prevOutData
 * @returns {*}
 */
SwitchNode.prototype.run = function ($line, $node, $prevOutData) {
    var node = $node;
    /**
     * 在end节点中
     */
    if ($line.context.end && $line.context.end.indexOf(node.id) > -1) {
        return Promise.resolve($prevOutData);
    }
    var isValid = this.valid(node.cases, $prevOutData);

    if (isValid === false) {
        return error.errorBuild(error.eErrorType.NotSetNodeNext, $line, $node);
    }

    if (util.isString(isValid) && isValid) {

        var nextNode = $line.context.nodes[isValid];

        if (!nextNode) {
            return error.errorBuild(error.eErrorType.NotFindNextNode, $line, $node);
        }

        return Promise.resolve($prevOutData)
            .then((function (_line, _node, _this, _prevNode) {
                return function (data) {
                    var result, typeNode;
                    switch (_node.type) {
                        case "switch":
                            result = _this.run(_line, _node, data);
                            break;
                        case "parallel":
                            typeNode = require("./node-parallel");
                            result = typeNode.run(_line, _node, data);
                            break;
                        case "combine":
                            typeNode = require("./node-combine");
                            result = typeNode.run(_line, _node, data, _prevNode);
                            break;
                        default :
                            typeNode = require("./node-process");
                            result = typeNode.run(_line, _node, data);
                            break;
                    }
                    if (result instanceof Error) {
                        return Promise.reject(result);
                    } else {
                        return result;
                    }
                };
            }($line, nextNode, this, node)))

    }
};

var instance = new SwitchNode();

module.exports = instance;