"use strict";

var util = require("util");
var fs = require("fs");

var eRunState = require("./model/po-eRunState");
var linePool = require("./busiFlow/linePool");
var store = require("./store");
var fileUtil = require("./utils/util-file");

/**
 * 对外输出接口
 * @type {{dao: (exports|module.exports), remoteMsg: (MsgRemoteIntf|exports|module.exports), busiNode: (BaseProcessor|exports|module.exports)}}
 */
exports.intf = {
    dao: require("./intf/dao"),
    remoteMsg: require("./intf/msg-remote"),
    busiNode: require("./intf/busi-node")
};

/**
 * options:please link to "https://github.com/buns-li/stream-coding"
 * @param options
 */
exports.load = function (options) {
    try {
        if (util.isString(options)) {
            var isJSON = fileUtil.jsonORjs(options);
            if (isJSON) {
                options = fs.readFileSync(options);
                options = JSON.parse(options);
            } else {
                options = require(options);
            }
        }
        store.store(options);
    }
    catch (e) {
        console.log(e);
    }
};

/**
 * 执行客户端发送过来的业务指令和业务指令携带的参数
 * @param directive
 * @param directiveArgDic
 * @param callback
 */
exports.run = function (directive, directiveArgDic, callback) {
    if (!directive) {
        throw new Error("请保证指令不为空");
    }
    if (!callback) {
        throw new Error("请保证回调不为空");
    }
    var cb = callback;
    var line = linePool.instance.get(directive, directiveArgDic);
    line.once('end', cb);
    if (line.state === eRunState.prepare) {
        line = line.run();
        line.promise
            .then(complete)
            .catch(errOutput);
    }

    function complete(data) {
        line.emit("completed", line);
        line.emit("end", null, data);
        line.removeAllListeners();
        return data;
    }

    function errOutput(err) {
        line.emit("completed", line);
        line.emit("end", err);
        line.removeAllListeners();
    }
};
