"use strict";

var strUtil = require("./utils/util-string");

function mapNodeType(nodeType) {
    switch (nodeType) {
        case"switch":
            return "条件判断";
        case "combine":
            return "合并";
        case "parallel":
            return "并行";
        default:
            return "处理器";
    }
}

var eErrorType = exports.eErrorType = {
    NotSetNodeNext: "Not Set Next Node Error",
    NotFindNode: "Not Find Node Error",
    NotFindNextNode: "Not Find Next Node Error",
    NotSetProcessor: "Not Set Processor Error",
    NotFindProcessor: "Not Find Processor Error",
    NotInputNodeRequireParam: "Not Input Node Require param Error",
    NotFindSqlKeys: "Not Find SqlKey Error",
    NoAccessToSqlKey: "No AccessRight To Current SqlKey",
    NotMatchSqlParams: "Not Match Sql Params"
};

exports.errorBuild = function (errType, $line, $node, $param) {
    var result;
    switch (errType) {
        case eErrorType.NotSetNodeNext:
            result = strUtil.formatStr("%1:业务线(%2)上的%3业务点(%4)没有设置next,导致业务线中断!", errType, $line.directive, mapNodeType($node.type), $node.id);
            break;
        case eErrorType.NotFindNode:
            result = strUtil.formatStr("%1:业务线(%2)上的%3业务点(%4)没有找到,导致业务线中断!", errType, $line.directive, mapNodeType($node.type), $node.id);
            break;
        case eErrorType.NotFindNextNode:
            result = strUtil.formatStr("%1:业务线(%2)上的%3业务点(%4)内有找到next节点(%5),导致业务线中断!", errType, $line.directive, mapNodeType($node.type), $node.id, $node.next);
            break;
        case eErrorType.NotSetProcessor:
            result = strUtil.formatStr("%1:业务线(%2)上的%3业务点(%4)没有设置处理器的文件地址,导致业务线中断!", errType, $line.directive, mapNodeType($node.type), $node.id);
            break;
        case eErrorType.NotFindProcessor:
            result = strUtil.formatStr("%1:业务线(%2)上的%3业务点(%4)没有找到对应的处理器(%5),导致业务线中断!", errType, $line.directive, mapNodeType($node.type), $node.id, $node.classPath);
            break;
        case eErrorType.NotInputNodeRequireParam:
            result = strUtil.formatStr("%1:业务线(%2)上的%3业务点(%4)接收的输入参数中必填属性(%5)未找到,导致业务线中断!", errType, $line.directive, mapNodeType($node.type), $node.id, $node.classPath, $param.key);
            break;
        case eErrorType.NotFindSqlKeys:
            result = strUtil.formatStr("%1:业务线(%2)上的处理器业务点(%3)在执行DAO操作的时候sqlKey配置未找到,导致业务线中断!", errType, $line, $node);
            break;
        case eErrorType.NoAccessToSqlKey:
            result = strUtil.formatStr("%1:业务线(%2)上的处理器业务点(%3)在执行DAO操作的时候sqlKey($4)与配置中的内容不匹配,造成无权限执行数据操作,导致业务线中断!", errType, $line, $node, $param);
            break;
        case eErrorType.NotMatchSqlParams:
            result = strUtil.formatStr("%1:业务线(%2)上的处理器业务点(%3)在执行DAO操作的时候sqlKey的参数(%4)缺失,造成数据操作异常，导致业务线中断!", errType, $line, $node, $param);
    }
    return new Error(result);
};