/**
 * 第三方插件调用的包装
 * Created by buns on 1/11/16.
 */

"use strict";

var url = require("url");
var http = require("http");
var https = require("https");
var qs = require("querystring");
var fs = require("fs");

var Promise = require("bluebird");
var iconv = require("iconv-lite");

var strUtil = require("../utils/util-string");

function ThirdIntf(opt) {
    this.ssl = opt.ssl;

    this.method = opt.method ? String(opt.method).toLowerCase() : "get";
    this.encode = opt.encode || "utf8";
    this.params = opt.params;
    this.url = opt.url;

    this.protocol = this.ssl ? https : http;
}

ThirdIntf.prototype.request = function request(clientParams) {
    var self = this;
    return new Promise(function (resolve, reject) {
        var postData = {};
        for (var param in self.params) {
            if (clientParams.hasOwnProperty(param.clientName)) {
                postData[param.name] = clientParams[param.clientName];
            }
        }
        postData = qs.stringify(postData);

        switch (self.method) {
            case "GET":
                self.url += strUtil.endWith(self.url, "?") ? postData : ("?" + postData);
                self.url = url.parse(url);
                break;
            case "POST":
                self.url = url.parse(self.url);
                self.url.agent = false;
                url.headers = {
                    'Content-Type': 'application/x-www-form-urlencoded',
                    'Content-Length': postData.length
                };
                break;
        }
        if (self.ssl) {
            self.url.key = fs.readFileSync(self.url.key);
            self.url.cert = fs.readFileSync(self.url.cert);
        }
        var req = self.protocol.request(self.url, function (res) {
            var datas = [];
            var size = 0;
            res.on('data', function (data) {
                datas.push(data);
                size += data.length;
            });
            res.on("end", function () {
                var buff = Buffer.concat(datas, size);
                var result = iconv.decode(buff, self.encode);
                resolve(result);
            });
        });

        if (self.method === "POST") {
            req.write(postData);
        }

        req.on('error', function (err) {
            reject(err);
        });

        req.end();
    });
};

module.exports = ThirdIntf;