"use strict";

var week = ["/u65e5", "/u4e00", "/u4e8c", "/u4e09", "/u56db", "/u4e94", "/u516d"]; // 一、二、三、四、五、六、七
var weekPrefix = {
    "1": "/u5468", //周
    "2": "/u661f/u671f" //星期
};
/**
 * 进行日期格式的转换
 * @param date
 * @param format
 * @returns {void|string|XML|*}
 */
exports.dateFormat = function (date, format) {
    var map = {
        "M": date.getMonth() + 1,
        "d": date.getDate(),
        "h": date.getHours(),
        "m": date.getMinutes(),
        "s": date.getSeconds(),
        "q": Math.floor((date.getMonth() + 3) / 3),
        "S": date.getMilliseconds()
    };
    format = format.replace(/([yMdhmsqSW])+/g, function (all, t) {
        var v = map[t];
        if (v !== undefined) {
            if (all.length > 1) {
                v = '0' + v;
                v = v.substr(v.length - 2);
            }
            return v;
        } else if (t === 'y') {
            return (date.getFullYear() + '').substr(4 - all.length);
        } else if (t === 'W') {
            return (weekPrefix[all.length + ""] || "") + week[date.getDay()];
        }
        return all;
    });

    return format;
};

/**
 *将时间字符串按照指定格式进行转换
 */
exports.formatToDate = (function (scope) {
    return function (date, format) {
        date = [date.substr(0, 4), date.substr(4, 2), date.substr(6, 2)].join('-') + " " + [date.substr(8, 2), date.substr(10, 2)].join(":");
        return scope.dateFormat(new Date(date), format);
    };
}(exports));

/**
 * 将时间字符串转换成14位的数字
 * @param date
 * @returns {string}
 */
exports.dateToLong = function (date) {
    var arr = [];
    arr.push(date.getFullYear());
    arr.push('0' + date.getMonth() + 1);
    arr.push('0' + date.getDate());
    arr.push('0' + date.getHours());
    arr.push('0' + date.getMinutes());
    arr.push('0' + date.getSeconds());
    for (var i = 1, l = arr.length; i < l; i++) {
        arr[i] = arr[i].substr(arr[i].length - 2);
    }
    return arr.join('');
};