"use strict";

var path = require("path");

var fs = require("fs");

var util = require("util");

var CONFIGFILE_REG = /.(js|json)/i;
/**
 * 判断是json文件还是js文件
 * @param file
 * @returns {*}
 */
var jsonORjs = exports.jsonORjs = function jsonORjs(file) {
    if (!file) return false;
    var ext = CONFIGFILE_REG.exec(file);
    return ext[1] === "json" ? 1 : 0;
};

/**
 * 读取文件夹下的所有文件(包含子目录)
 * @param dir 文件夹目录
 * @param suffix 允许读取的文件后缀
 * @param content
 */
function readDirFiles(dir, suffix, content) {
    var configContent;
    fs.readdirSync(dir).forEach(function (file) {
        file = path.resolve(dir, file);
        var stat = fs.statSync(file);
        if (stat.isDirectory()) {
            readDirFiles(file, suffix, content);
        } else {
            var ext = path.extname(file);
            if (suffix.indexOf(ext) === -1) return;
            var isJSON = jsonORjs(file);
            if (isJSON) {
                configContent = fs.readFileSync(file);
                contents.push(JSON.parse(configContent));
            } else {
                content.push(require(file));
            }
        }
    });
}

/**
 * 加载配置文件
 * @param filePath
 * @param suffix 允许读取的文件后缀
 * @returns {Array}
 */
exports.loadConfigFile = function loadConfigFile(filePath, suffix) {

    if (!suffix)suffix = ".js|.json";

    var contents = [];
    if (util.isArray(filePath)) {
        filePath.forEach(function () {
            contents.push(fs.readFileSync(filePath));
        });
    } else {
        var configContent;
        var stat = fs.statSync(filePath);
        if (stat && stat.isDirectory()) {
            readDirFiles(filePath, suffix, contents);
        } else {
            var ext = path.extname(filePath);
            if (suffix.indexOf(ext) === -1) return contents;
            var isJSON = jsonORjs(filePath);
            if (isJSON) {
                configContent = fs.readFileSync(filePath);
                contents.push(JSON.parse(configContent));
            } else {
                contents.push(require(filePath));
            }
        }
    }
    return contents;
};
