"use strict";

var util = require('util');
/**
 * 执行两个对象的比较
 * @param $old
 * @param $new
 * @return boolean 相同:true 不相同:false
 */
exports.compare = function compare($old, $new) {
    //使用 === 来保证了类型和值都相同的情况：比较String,Boolean,Number,null,undefined,Object
    if ($old === $new)
        return true;
    //保证都是对象的前提
    if (!($old instanceof Object && $new instanceof Object))
        return false;
    //保证两个对象的都是一个原型链的
    if ($old.constructor !== $new.constructor)
        return false;

    var result;

    if (util.isArray($old) && util.isArray($new)) {
        if ($old.length !== $new.length) return false;
        var l = $old.length;
        for (; l--;) {
            result = compare($old[l], $new[l]);
            if (!result) return false;
        }
    } else {
        for (var i in $old) {
            if ($old.hasOwnProperty(i) && !$new.hasOwnProperty(i)) return false;
            result = compare($old[i], $new[i]);
            if (!result) return false;
        }

        for (i in $new) {
            if ($new.hasOwnProperty(i) && !$old.hasOwnProperty(i)) return false;
        }
    }
    return result;
};

var copy = exports.copy = function (obj) {
    var result;
    if (!obj) return null;
    result = {};
    for (var i in obj) {
        if (!obj.hasOwnProperty(i)) continue;
        result[i] = util.isObject(obj[i]) ? copy(obj[i]) : obj[i];
    }
    return result;
};